#include <stdio.h>
#include <stdlib.h>
#include <mex.h>

#include "../headers/matrix.h"
#include "../headers/circleFit.h"
#include "../headers/mexConversion.h"

/***************************************************************/
/***************************************************************/
/***************************************************************/
int findMaximumCircleRadius(MATRIX circles){
    int maxR = circles.data[0][2], i;
    
    for (i = 0; i < circles.row; i++)
        if (circles.data[i][2] > maxR)
            maxR = circles.data[i][2];
    
    return maxR;
}
MATRIX convertCircles2CircleMap(MATRIX circles, int row, int column){
    MATRIX map       = allocateMatrix(row,column);
    int maxR         = findMaximumCircleRadius(circles), i, j, k, cx, cy, cr, incrx, incry;
    CIRCLE_BND *cbnd = createCircularBoundaries(maxR + 1);

    initializeMatrix(&map,-1);
    for (i = 0; i < circles.row; i++){
        cx = circles.data[i][0];
        cy = circles.data[i][1];
        cr = circles.data[i][2];
        
        map.data[cx][cy] = i;
        for (j = 0; j <= cr; j++){
            for (k = 0; k < cbnd[j].N; k++){
                incrx = cbnd[j].x[k];
                incry = cbnd[j].y[k];
                
                map.data[cx+incrx][cy+incry] = i;
                map.data[cx+incrx][cy-incry] = i;
                map.data[cx-incrx][cy+incry] = i;
                map.data[cx-incrx][cy-incry] = i;
            }
        }
    }
    freeCircularBoundaries(cbnd,maxR + 1);
    return map;
}
/***************************************************************/
/***************************************************************/
/***************************************************************/
/* It locates the circles on the 1 pixels                      */
/*                                                             */
/* Outputs: circles and circleMap                              */
/*                                                             */
/* Circles are in the format of the following                  */
/*          circles(i,0) = xCoor                               */
/*          circles(i,1) = yCoor                               */
/*          circles(i,2) = radius                              */
/*          circles(i,3) = label (1)                           */
/*                                                             */
/* circleMap has -1 pixels for the background and it labels    */
/*          the circles from 0 to N-1                          */
/***************************************************************/
/***************************************************************/
/***************************************************************/
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){

    MATRIX cmap      = convertMxArray2Matrix(prhs[0]);
    int minRadius    = (int)(mxGetScalar(prhs[1]));
    MATRIX circles   = locateCircles(cmap,1,minRadius,-1);
    MATRIX circleMap = convertCircles2CircleMap(circles,cmap.row,cmap.column);
    
    plhs[0]         = convertMatrix2MxArray(circles);
    plhs[1]         = convertMatrix2MxArray(circleMap);
    
    freeMatrix(cmap);
    freeMatrix(circles);
    freeMatrix(circleMap);
}












