% This function uses the code provided by Geoff Hinton and Ruslan Salakhutdinov 
% It restructured the statements in this code
%
%
%
% Version 1.000 
%
% Code provided by Geoff Hinton and Ruslan Salakhutdinov 
%
% Permission is granted for anyone to copy, use, modify, or distribute this
% program and accompanying programs and documents for any purpose, provided
% this copyright notice is retained and prominently displayed, along with
% a note saying that the original programs are available from our
% web page.
% The programs and documents are distributed without any warranty, express or
% implied.  As the programs were written for research purposes only, they have
% not been tested to the degree that would be advisable in any important
% application.  All use of these programs is entirely at the user's own
% risk.

% This program trains Restricted Boltzmann Machine in which
% visible, binary, stochastic pixels are connected to
% hidden, binary, stochastic feature detectors using symmetrically
% weighted connections. Learning is done with 1-step Contrastive Divergence.   
% The program assumes that the following variables are set externally:
% maxepoch  -- maximum number of epochs
% numhid    -- number of hidden units 
% batchdata -- the data that is divided into batches (numcases numdims numbatches)
% restart   -- set to 1 if learning starts from beginning 

function [W, A, B, batchposP] = rbmFunc(maxEpoch, hiddenNo, batchData)

epsW        = 0.01;   % Learning rate for weights 
epsA        = 0.01;   % Learning rate for biases of visible units 
epsB        = 0.01;   % Learning rate for biases of hidden units    
wCost       = 0.0002; 
initMom     = 0.5;
finalMom    = 0.9;

[sampleNo, dimNo, batchNo]  = size(batchData); 
    
W           = 0.1 * randn(dimNo, hiddenNo);
B           = zeros(1, hiddenNo);
A           = zeros(1, dimNo);
deltaW      = zeros(dimNo,hiddenNo);
deltaB      = zeros(1,hiddenNo);
deltaA      = zeros(1,dimNo);
batchposP   = zeros(sampleNo, hiddenNo, batchNo);

for epoch = 1 : maxEpoch
  
    if epoch > 5
        mom = finalMom;
    else
        mom = initMom;
    end
    
    errSum = 0;    
    for batch = 1 : batchNo
        
        V       = batchData(:,:,batch);
        repB    = repmat(B, sampleNo, 1);
        repA    = repmat(A, sampleNo, 1);
        
        posP    = 1 ./ (1 + exp(-( V * W + repB )));
        hidden  = posP > rand(sampleNo, hiddenNo);        
        negV    = 1 ./ (1 + exp(-( hidden * W' + repA ))); % bu application'a gore secilmeli
        negP    = 1 ./ (1 + exp(-( negV * W + repB )));
        
        deltaW  = mom * deltaW + epsW * ((V' * posP - negV' * negP) / sampleNo - wCost * W);
        deltaA  = mom * deltaA + epsA * ( sum(V) - sum(negV) ) / sampleNo;
        deltaB  = mom * deltaB + epsB * ( sum(posP) - sum(negP) ) / sampleNo;
    

        W = W + deltaW;
        A = A + deltaA;
        B = B + deltaB;
    
        batchposP(:,:,batch) = posP;
        errSum = errSum + sum(sum((V - negV) .^ 2));                
    end
    
    errSum = errSum / (sampleNo * batchNo);
    fprintf('epoch %d: %8.1f\n', epoch, errSum);
    
end

end
% ------------------------------------------------------- %
% ------------------------------------------------------- %
% ------------------------------------------------------- %
