function newIm = stainDeconvolution(im, method)

if nargin == 1
    method = 'H&E';
end

M = initialCoefficientMatrix(method);
D = findDeconvolutionMatrix(M);
newIm = deconvolveImage(im,D);

end
% --------------------------------------------------- %
% ---------- Find the deconvolution matrix ---------- %
% --------------------------------------------------- %
function D = findDeconvolutionMatrix(M)

% normalize the M matrix
R = zeros(3,3);
len = sqrt(sum(M .* M));
for i = 1 : 3
    if (len(i) > 0)
        R(:,i) = M(:,i) / len(i);
    end
end
% find values for the 3rd color if it is not specified
if (len(3) == 0)
    rsum = sum(R(:,1:2) .* R(:,1:2),2);
    for i = 1 : 3
        if (rsum(i) < 1)
            R(i,3) = sqrt(1 - rsum(i));
        end
    end
    len3 = sqrt(R(:,3)' * R(:,3));
    R(:,3) = R(:,3) / len3;
end
D = inv(R);

end
% --------------------------------------------------- %
% ---- Deconvolve the log transform of the image ---- %
% --------------------------------------------------- %
function newIm = deconvolveImage(im,D)

log255 = log(255);
[dx,dy] = size(im(:,:,1));
newIm = 255 * (1 - log(double(im) + 1) / log255);
newIm = reshape(newIm,[dx*dy 3])';
newIm = (D * newIm)';
newIm = uint8(round(255.^(1 - newIm/255)));
newIm = reshape(newIm,[dx dy 3]);

end
% --------------------------------------------------- %
% --------------------------------------------------- %
% --------------------------------------------------- %
function M = initialCoefficientMatrix(method)

    switch method
        case {'H&E'}
            %       GL Haem         GL Eos          Zero
            M = [   0.644211        0.092789        0.0
                    0.716556        0.954111        0.0
                    0.266844        0.283111        0.0     ];
       
        case { 'H&E2'}
            %       GL Haem         GL Eos          Zero
            M = [   0.49015734      0.04615336      0.0
                    0.76897085      0.8420684       0.0
                    0.41040173      0.5373925       0.0     ];
        
        case { 'H DAB' }    % 3,3-diamino-benzidine tetrahydrochloride
            %       GL Haem         DAB             Zero
            M = [   0.650           0.268           0.0
                    0.704           0.570           0.0
                    0.286           0.776           0.0     ];
        case { 'Feulgen LightGreen' }
            %       Feulgen         LightGreen      Zero
            M = [   0.46420921      0.94705542      0.0
                    0.83008335      0.25373821      0.0
                    0.30827187      0.19650764      0.0     ];
        case { 'Giemsa'}
            %       GL  MB&Eosin    GL Eos          Zero
            M = [   0.834750233     0.092789        0.0
                    0.513556283     0.954111        0.0
                    0.196330403     0.283111        0.0     ];
        case { 'FastRed FastBlue DAB' }
            %       FastRed         FastBlue        DAB
            M = [   0.21393921      0.74890292      0.268
                    0.85112669      0.60624161      0.570
                    0.47794022      0.26731082      0.776   ];
        case { 'MethylGreen DAB' }
            %       MG              DAB             Zero
            M = [   0.98003         0.268           0.0
                    0.144316        0.570           0.0
                    0.133146        0.776           0.0     ];
        case { 'H&E DAB' }
            %       Haem            Eos             DAB
            M = [   0.650           0.072           0.268
                    0.704           0.990           0.570
                    0.286           0.105           0.776   ];
        case { 'H AEC' }
            %       Haem            AEC             Zero
            M = [   0.650           0.2743          0.0
                    0.704           0.6796          0.0
                    0.286           0.6803          0.0     ];
        case { 'H PAS' }
            %       R               G               Zero
            M = [   0.644211        0.175411        0.0
                    0.716556        0.972178        0.0
                    0.266844        0.154589        0.0     ];
        case { 'RGB' }
            %       R               G               B
            M = [   0.0             1.0             1.0
                    1.0             0.0             1.0
                    1.0             1.0             0.0     ];
        case { 'CMY' }
            %       C               M               Y
            M = [   1.0             0.0             0.0
                    0.0             1.0             0.0
                    0.0             0.0             1.0     ];
    end
end
            
		
		

		

		

		

		

		

		
		

		


		


